@@ -30,17 +30,6 @@ import sys
 import tomcatmanager as tm
 
 
-def _positive_integer(value):
-    msg = "'{}' is not an integer greater than or equal to 0".format(value)
-    try:
-        ivalue = int(value)
-        if ivalue < 0:
-            raise argparse.ArgumentTypeError(msg)
-    except ValueError as valerror:
-        raise argparse.ArgumentTypeError(msg) from valerror
-    return ivalue
-
-
 def _build_parser():
     """Build the argument parser"""
     parser = argparse.ArgumentParser(
@@ -55,7 +44,7 @@ def _build_parser():
     parser.add_argument("-p", "--password", help=password_help)
 
     timeout_help = "timeout (in seconds) for network requests"
-    parser.add_argument("-t", "--timeout", type=_positive_integer, help=timeout_help)
+    parser.add_argument("-t", "--timeout", type=float, help=timeout_help)
 
     echo_help = "echo the command into the output"
     parser.add_argument("-e", "--echo", action="store_true", help=echo_help)
@@ -96,8 +85,15 @@ def _build_parser():
 
 #
 # entry point for command line
+# pylint: disable=too-many-branches
 def main(argv=None):
-    """Entry point for 'tomcat-manager' command line program."""
+    """Entry point for 'tomcat-manager' command line program.
+
+    :param argv:   pass a list of arguments to be processed. If None, sys.argv[1:] will
+                   be used. To process with no arguments, pass an empty list.
+    """
+    if argv is None:
+        argv = sys.argv[1:]
     parser = _build_parser()
     args = parser.parse_args(argv)
     if args.debug:
@@ -116,7 +112,9 @@ def main(argv=None):
         itm.status_to_stdout = True
     if args.debug:
         itm.debug = True
-    if args.timeout:
+    # have to check None becasue the timeout could be zero, which would evaluate
+    # to false, and in this case 0 is a valid timeout that we want to set
+    if args.timeout is not None:
         itm.timeout = args.timeout
 
     if args.manager_url:
@@ -144,5 +142,5 @@ def main(argv=None):
     return itm.exit_code
 
 
-if __name__ == "__main__":
+if __name__ == "__main__":  # pragma: nocover
     sys.exit(main())
