@@ -21,6 +21,8 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 #
+# pylint: disable=protected-access, missing-function-docstring
+# pylint: disable=missing-module-docstring, unused-variable
 
 import io
 import sys
@@ -40,6 +42,30 @@ def test_main_noargs(mocker):
     assert mock_cmdloop.call_count == 1
 
 
+def test_main_sys_argv(tomcat_manager_server, capsys, monkeypatch):
+    monkeypatch.setattr(
+        "sys.argv",
+        [
+            "tomcat-manager",
+            "-u",
+            tomcat_manager_server.user,
+            "-p",
+            tomcat_manager_server.password,
+            tomcat_manager_server.url,
+            "list",
+        ],
+    )
+
+    exit_code = main()
+    out, err = capsys.readouterr()
+    out = out.splitlines()
+    err = err.splitlines()
+    assert exit_code == 0
+    assert "Path" in out[0]
+    assert "Sessions" in out[0]
+    assert "--connected to" in err[0]
+
+
 def test_main_user_password_url_command(tomcat_manager_server, capsys):
     cmdline = "-u {} -p {} {} list".format(
         tomcat_manager_server.user,
@@ -191,7 +217,7 @@ def test_main_status_to_stdout(tomcat_manager_server, capsys):
 
 
 def test_main_timeout(tomcat_manager_server, capsys):
-    cmdline = "-t 25 -u {} -p {} {} settings timeout".format(
+    cmdline = "-t 7.8 -u {} -p {} {} settings timeout".format(
         tomcat_manager_server.user,
         tomcat_manager_server.password,
         tomcat_manager_server.url,
@@ -201,4 +227,18 @@ def test_main_timeout(tomcat_manager_server, capsys):
     out, _ = capsys.readouterr()
     out = out.splitlines()
     assert exit_code == 0
-    assert "timeout=25" in out[0]
+    assert "timeout=7.8" in out[0]
+
+
+def test_main_timeout_zero(tomcat_manager_server, capsys):
+    cmdline = "-t 0 -u {} -p {} {} settings timeout".format(
+        tomcat_manager_server.user,
+        tomcat_manager_server.password,
+        tomcat_manager_server.url,
+    )
+    argv = cmdline.split(" ")
+    exit_code = main(argv)
+    out, _ = capsys.readouterr()
+    out = out.splitlines()
+    assert exit_code == 0
+    assert "timeout=0.0" in out[0]
